class StockGui(object):
    def __init__(self):
        import PySimpleGUI as sg
        self.sg = sg
        self.sg.theme('BlueMono')
        layout = [
            [self.sg.Button('Markovitz Portfolio')],
            [self.sg.Button('Get Good Stocks')],
            [self.sg.Button('Get Stock Data')],
            [self.sg.Button('Get Stock Capacity')],
            [self.sg.Button('Stock Industry and Constituent Stock')],
        ]
        self.window = self.sg.Window('Suluoya Stock', layout)
        self.event, self.values = self.window.read()

    def MarkovitzGui(self):

        layout = [
            [self.sg.Text('Start Date'), self.sg.Input('2019-01-01')],

            [self.sg.Text(' End Date'), self.sg.Input('2020-01-01')],

            [self.sg.Text('Frequency'),
                self.sg.Radio('day', 'Frequency', default=True),
                self.sg.Radio('week', 'Frequency', default=False),
                self.sg.Radio('month', 'Frequency', default=False)],

            [self.sg.Text('Stock List')],
            [self.sg.Multiline('贵州茅台\n隆基股份\n五粮液')],

            [self.sg.Text('Holiday Mode')],
            [self.sg.Radio('open', 'Holiday Mode', default=False),
             self.sg.Radio('close', 'Holiday Mode', default=True)],

            [self.sg.Text('Holiday Name')],
            [self.sg.Combo(['国庆节', '中秋节', '春节'], default_value='春节')],

            [self.sg.Text('before')],
            [self.sg.Input('-21')],

            [self.sg.Text('after')],
            [self.sg.Input('21')],

            [self.sg.Text('risk-free interest rate (annually)')],
            [self.sg.Input('0.0185')],

            [self.sg.Text('Calculate Mode')],
            [self.sg.Radio('iteration algorithm', 'Calculate Mode', default=True),
                self.sg.Radio('generate scatters', 'Calculate Mode', default=False), ],

            [self.sg.Text('Scatter Number')],
            [self.sg.Input('500')],

            [self.sg.Button('Start working!')],
        ]

        window = self.sg.Window('Suluoya Markovitz', layout)

        event, values = window.read()
        if event == 'Cancel' or event is None:
            window.close()
            return 'Quit'
        names = values[5].split('\n')[:-1]
        start_date = values[0]
        end_date = values[1]

        if values[2]:
            frequency = 'd'
        if values[3]:
            frequency = 'w'
        if values[4]:
            frequency = 'm'

        if values[6]:
            holiday = True
        if values[7]:
            holiday = False

        holiday_name = values[8]

        before = values[9]
        after = values[10]

        no_risk_rate = float(values[11])

        if values[12]:
            accurate = True
        if values[13]:
            accurate = False

        number = int(values[14])

        window.close()

        try:
            from .Markovitz import Markovitz
        except:
            from Markovitz import Markovitz

        Markovitz = Markovitz(names=names,
                              start_date=start_date,
                              end_date=end_date,
                              frequency=frequency,
                              holiday=holiday,
                              holiday_name=holiday_name,
                              before=before, after=after,
                              no_risk_rate=no_risk_rate
                              )
        print(Markovitz.portfolio(accurate=accurate, number=number))

    def MarkovitzWork(self):
        if self.event == 'Markovitz Portfolio':
            self.window.close()
            self.MarkovitzGui()

    def StockDataGui(self):
        layout = [
            [self.sg.Text('Start Date'), self.sg.Input(
                '2019-01-01', key='start_date')],

            [self.sg.Text(' End Date'), self.sg.Input(
                '2020-01-01', key='end_date')],

            [self.sg.Text('Frequency'),
                self.sg.Radio('day', 'Frequency', default=True, key='day'),
                self.sg.Radio('week', 'Frequency', default=False, key='week'),
                self.sg.Radio('month', 'Frequency', default=False, key='month')],

            [self.sg.Text('Stock List')],
            [self.sg.Multiline('贵州茅台\n隆基股份\n五粮液', key='stock_list')],

            [self.sg.Text('Holiday Mode')],
            [self.sg.Radio('open', 'Holiday Mode', default=False, key='open'),
             self.sg.Radio('close', 'Holiday Mode', default=True, key='close')],

            [self.sg.Text('Holiday Name')],
            [self.sg.Combo(['国庆节', '中秋节', '春节'],
                           default_value='春节', key='holiday')],

            [self.sg.Text('before')],
            [self.sg.Input('-21', key='before')],

            [self.sg.Text('after')],
            [self.sg.Input('21', key='after')],

            [self.sg.FolderBrowse('choose a folder to save data', key='path')],

            [self.sg.Button('Start working!')]
        ]
        window = self.sg.Window('Suluoya Stock Data', layout)
        event, values = window.read()
        if event == 'Cancel' or event is None:
            return 'Quit'
        window.close()
        start_date = values['start_date']
        end_date = values['end_date']
        if values['day']:
            frequency = 'd'
        if values['week']:
            frequency = 'w'
        if values['month']:
            frequency = 'm'
        stock_list = values['stock_list'].rstrip().split('\n')
        path = values['path']
        if path == '':
            raise ValueError('Please choose a folder to save data!')

        if values['close']:
            try:
                from .GetData import StockData
            except:
                from GetData import StockData
            StockData = StockData(names=stock_list,
                                  start_date=start_date, end_date=end_date,
                                  frequency=frequency)
            stock_pair, stock_data = StockData.stock_data[1:]
            StockData.quit()

        if values['open']:
            try:
                from .GetData import HolidayStockData
            except:
                from GetData import HolidayStockData
            holiday = values['holiday']
            before = values['before']
            after = values['after']
            HolidayStockData = HolidayStockData(names=stock_list,
                                                start_date=start_date,
                                                end_date=end_date,
                                                frequency=frequency,
                                                holiday=holiday,
                                                before=before,
                                                after=after)
            stock_pair, stock_data = HolidayStockData.HolidayNearbyData[1:]
        try:
            from ..log.SlyLog import sprint
        except:
            from log.SlyLog import sprint
        sprint = sprint()
        sprint.cyan('\n'+str(stock_pair))
        stock_data.to_excel(f'{path}\\stock_data.xlsx',
                            index=False, encoding='utf8')

    def StockDataWork(self):
        if self.event == 'Get Stock Data':
            self.window.close()
            self.StockDataGui()

    def StockCapacityGui(self):
        layout = [
            [self.sg.Text('Stock List')],
            [self.sg.Multiline('贵州茅台\n隆基股份\n五粮液', key='stock_list')],

            [self.sg.Text('        start                 end')],
            [self.sg.Text('  year    quater    year    quater')],
            [self.sg.Combo(values=[str(i) for i in range(2007, 2021)], default_value=2018, key='start_year'),
             self.sg.Combo(values=[str(i) for i in range(
                 1, 5)], default_value=1, key='start_quater'),
             self.sg.Combo(values=[str(i) for i in range(
                 2007, 2021)], default_value=2019, key='end_year'),
             self.sg.Combo(values=[str(i) for i in range(1, 5)], default_value=4, key='end_quater')],

            [self.sg.Text('choose capacities')],
            [self.sg.Checkbox('profit', default=True, key='profit'),
             self.sg.Checkbox('operation', default=True, key='operation'),
             self.sg.Checkbox('growth', default=True, key='growth'),
             self.sg.Checkbox('balance', default=True, key='balance'),
             self.sg.Checkbox('cash flow', default=True, key='cash'),
             self.sg.Checkbox('dupont', default=True, key='dupont'), ],

            [self.sg.FolderBrowse('choose a folder to save data', key='path')],

            [self.sg.Button('Start working!')]
        ]
        window = self.sg.Window('Suluoya Stock Capacity', layout)
        event, values = window.read()
        if event == 'Cancel' or event is None:
            window.close()
            return 'Quit'
        window.close()
        stock_list = values['stock_list'].rstrip().split('\n')
        start_year = int(values['start_year'])
        end_year = int(values['end_year'])
        start_quater = int(values['start_quater'])
        end_quater = int(values['end_quater'])
        path = values['path']
        if path == '':
            raise ValueError('Please choose a folder to save data!')
        try:
            from .GetData import StockAbility
        except:
            from GetData import StockAbility
        sa = StockAbility(names=stock_list,
                          start_year=start_year, start_quater=start_quater,
                          end_year=end_year, end_quater=end_quater)
        if values['profit'] and values['operation'] and values['growth'] and values['balance'] and values['cash'] and values['dupont']:
            sa.AllAbility.to_excel(f'{path}\\stock_capacity.xlsx',
                                   index=False, encoding='utf8')
        else:
            result_list = []
            if values['profit']:
                result_list.append(sa.profit)
            if values['operation']:
                result_list.append(sa.operation)
            if values['growth']:
                result_list.append(sa.growth)
            if values['balance']:
                result_list.append(sa.balance)
            if values['cash']:
                result_list.append(sa.cash_flow)
            if values['dupont']:
                result_list.append(sa.dupont_data)
            from pandas import merge
            if len(result_list) == 2:
                df = merge(result_list[0], result_list[1], how='outer', on=[
                    'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
            elif len(result_list) == 1:
                df = result_list[0]
            else:
                df = merge(result_list[0], result_list[1], how='outer', on=[
                    'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
                for i in result_list[2:]:
                    df = merge(df, i, how='outer', on=[
                        'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
            df.to_excel(f'{path}\\stock_capacity.xlsx',
                        index=False, encoding='utf8')

    def StockCapacityWork(self):
        if self.event == 'Get Stock Capacity':
            self.window.close()
            self.StockCapacityGui()

    def StockIndustryGui(self):
        layout = [
            [self.sg.FolderBrowse('choose a folder to save data', key='path')],
            [self.sg.Button('sz50'), self.sg.Button(
                'hs300'), self.sg.Button('zz500')],

            [self.sg.Text('Stock List')],
            [self.sg.Multiline('贵州茅台\n隆基股份\n五粮液', key='stock_list')],
            [self.sg.Button('Start working!')]
        ]
        window = self.sg.Window('Suluoya Stock Industry', layout)
        try:
            from .GetData import ConstituentStock
        except:
            from GetData import ConstituentStock

        cs = ConstituentStock(save=False)
        first = True
        while 1:
            try:
                event, values = window.read(timeout=10)
                path = values['path']
                if path == '':
                    continue
                if event == 'Cancel' or event is None:
                    window.close()
                    return 'Quit'
                elif event == 'sz50':
                    if not first:
                        try:
                            from ..log.SlyLog import sprint
                        except:
                            from log.SlyLog import sprint
                        sprint = sprint()
                        sprint.hide()
                        cs.bs.login()
                        sprint.show()
                    cs.sz50.to_excel(f'{path}\\sz50.xlsx',
                                     index=False, encoding='utf8')
                    print(f'save in {path}\\sz50.xlsx')
                    first = False
                elif event == 'hs300':
                    if not first:
                        try:
                            from ..log.SlyLog import sprint
                        except:
                            from log.SlyLog import sprint
                        sprint = sprint()
                        sprint.hide()
                        cs.bs.login()
                        sprint.show()
                    cs.sz50.to_excel(f'{path}\\hs300.xlsx',
                                     index=False, encoding='utf8')
                    print(f'save in {path}\\hs300.xlsx')
                    first = False
                elif event == 'zz500':
                    if not first:
                        try:
                            from ..log.SlyLog import sprint
                        except:
                            from log.SlyLog import sprint
                        sprint = sprint()
                        sprint.hide()
                        cs.bs.login()
                        sprint.show()
                    cs.sz50.to_excel(f'{path}\\zz500.xlsx',
                                     index=False, encoding='utf8')
                    print(f'save in {path}\\zz500.xlsx')
                    first = False
                elif event == 'Start working!':
                    if not first:
                        try:
                            from ..log.SlyLog import sprint
                        except:
                            from log.SlyLog import sprint
                        sprint = sprint()
                        sprint.hide()
                        cs.bs.login()
                        sprint.show()
                    df = cs.StockIndustry(
                        names=values['stock_list'].rstrip().split('\n'))
                    print(df)
                    df.to_excel(f'{path}\\industry.xlsx',
                                index=False, encoding='utf8')
                    print(f'\nsave in {path}\\industry.xlsx')
                    first = False
            except:
                window.close()
                break

    def StockIndustryWork(self):
        if self.event == 'Stock Industry and Constituent Stock':
            self.window.close()
            self.StockIndustryGui()

    def GoodStockGui(self):
        layout = [
            [self.sg.Text('page')],
            [self.sg.Combo(list(range(1, 21)),default_value=3,key='page')],
            [self.sg.FolderBrowse('choose a folder to save data', key='path')],
            [self.sg.Button('Start working!')],
        ]
        window = self.sg.Window('Suluoya Good Stocks', layout)
        event, values = window.read()
        if event == 'Cancel' or event is None:
            window.close()
            return 'Quit'
        window.close()
        path = values['path']
        if path == '':
            raise ValueError('Please choose a folder to save data!')
        page=int(values['page'])
        try:
            from .GetGoodStock import GetGoodStock
        except:
            from GetGoodStock import GetGoodStock
        df=GetGoodStock(page=page)
        df.to_excel(f'{path}\\good_stocks.xlsx',encoding='utf8')
        
    def GoodStockWork(self):
        if self.event == 'Get Good Stocks':
            self.window.close()
            self.GoodStockGui()
def gui():
    sg = StockGui()
    while 1:
        a = sg.MarkovitzWork()
        c = sg.StockCapacityWork()
        b = sg.StockDataWork()
        d = sg.StockIndustryWork()
        e = sg.GoodStockWork()
        if a == None and b == None and c == None and d == None and e == None:
            break    


if __name__ == '__main__':
    sg = StockGui()
    while 1:
        a = sg.MarkovitzWork()
        c = sg.StockCapacityWork()
        b = sg.StockDataWork()
        d = sg.StockIndustryWork()
        e = sg.GoodStockWork()
        if a == None and b == None and c == None and d == None and e == None:
            break





class svg_charts(object):

    from pygal.style import BlueStyle, CleanStyle, DarkColorizedStyle, DarkGreenBlueStyle, DarkGreenStyle, DarkSolarizedStyle, DarkStyle, DarkenStyle, DefaultStyle, DesaturateStyle, LightColorizedStyle, LightGreenStyle, LightSolarizedStyle, LightStyle, LightenStyle, NeonStyle, ParametricStyleBase, RedBlueStyle, RotateStyle, SaturateStyle, SolidColorStyle, Style, TurquoiseStyle
    def __init__(self,
                 width=1440,
                 height=720,
                 spacing=30,
                 margin=50,
                 style=BlueStyle,
                 ):
        self.width = width
        self.height = height
        self.spacing = spacing
        self.margin = margin
        self.style = style

    def bar(self, x=[1, 2, 3, 4, 5, 6],
            data={'A': [0, 1, 1, 2, 3, 5], 'B': [3, 4, 5, 7, 9, 1]}):
        from pygal import Bar
        chart = Bar(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title='Suluoya-svg',
            x_title="I'm xlabel",
            y_title="I'm ylabel",
            show_x_labels=True,
            show_x_guides=False,
            x_label_rotation=0,  # 倾斜x轴标签
            show_y_labels=True,
            show_y_guides=False,
            y_label_rotation=0,  # 倾斜y轴标签
            print_values=True,
            print_values_position='top',  # bottom,
            print_zeroes=False,
            show_legend=True,
            legend_at_bottom=False,
            include_x_axis=False,  # 包含x=0
            inverse_y_axis=False,
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            rounded_bars=False,
        )
        chart.x_labels = x
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def horizontal_bar(self, x=[1, 2, 3, 4, 5, 6],
                       data={'A': [0, 1, 1, 2, 3, 5], 'B': [3, 4, 5, 7, 9, 1]}):
        from pygal import HorizontalBar
        chart = HorizontalBar(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title='Suluoya-svg',
            x_title="I'm xlabel",
            y_title="I'm ylabel",
            show_x_labels=True,
            show_x_guides=False,
            x_label_rotation=0,  # 倾斜x轴标签
            show_y_labels=True,
            show_y_guides=False,
            y_label_rotation=0,  # 倾斜y轴标签
            print_values=True,
            print_values_position='top',  # bottom,
            print_zeroes=False,
            show_legend=True,
            legend_at_bottom=False,
            include_x_axis=False,  # 包含x=0
            inverse_y_axis=False,
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            rounded_bars=False,
        )
        chart.x_labels = x
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def stacked_bar(self, x=[1, 2, 3, 4, 5, 6],
                    data={'A': [0, 1, 1, 2, 3, 5], 'B': [3, 4, 5, 7, 9, 1]}):
        from pygal import StackedBar
        chart = StackedBar(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title='Suluoya-svg',
            x_title="I'm xlabel",
            y_title="I'm ylabel",
            show_x_labels=True,
            show_x_guides=False,
            x_label_rotation=0,  # 倾斜x轴标签
            show_y_labels=True,
            show_y_guides=False,
            y_label_rotation=0,  # 倾斜y轴标签
            print_values=True,
            print_values_position='top',  # bottom,
            print_zeroes=False,
            show_legend=True,
            legend_at_bottom=False,
            include_x_axis=False,  # 包含x=0
            inverse_y_axis=False,
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            rounded_bars=False,
        )
        chart.x_labels = x
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def line(self, x=[1, 2, 3, 4, 5, 6],
            data={'A': [0, 1, 1, 2, 3, 5], 'B': [3, 4, 5, 7, 9, 1]},
            title='Suluoya-svg',
            x_title="I'm xlabel",
            y_title="I'm ylabel",
            show_x_labels=True,
            show_x_guide=False,
            x_label_rotation=0,  # 倾斜x轴标签
            show_y_labels=True,
            show_y_guides=False,
            y_label_rotation=0,  # 倾斜y轴标签
            print_values=True,
            print_zeroes=False,
            show_legend=True,
            legend_at_bottom=False,
            include_x_axis=False,  # 包含x=0
            inverse_y_axis=False,
            interpolation='cubic',  # 折线图平滑处理
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            fill=False):
        from pygal import Line
        chart = Line(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title=title,
            x_title=x_title,
            y_title=y_title,
            show_x_labels=show_x_labels,
            show_x_guide=show_x_guide,
            x_label_rotation=x_label_rotation,  # 倾斜x轴标签
            show_y_labels=show_y_labels,
            show_y_guides=show_y_guides,
            y_label_rotation=y_label_rotation,  # 倾斜y轴标签
            print_values=print_values,
            print_zeroes=print_zeroes,
            show_legend=show_legend,
            legend_at_bottom=legend_at_bottom,
            include_x_axis=include_x_axis,  # 包含x=0
            inverse_y_axis=inverse_y_axis,
            interpolation=interpolation,  # 折线图平滑处理
            value_formatter=value_formatter,
            human_readable=human_readable,
            fill=fill,  # 面积图
        )
        chart.x_labels = x
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def stacked_line(self, x=[1, 2, 3, 4, 5, 6],
                     data={'A': [0, 1, 1, 2, 3, 5], 'B': [3, 4, 5, 7, 9, 1]}):
        from pygal import StackedLine
        chart = StackedLine(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title='Suluoya-svg',
            x_title="I'm xlabel",
            y_title="I'm ylabel",
            show_x_labels=True,
            show_x_guide=False,
            x_label_rotation=0,  # 倾斜x轴标签
            show_y_labels=True,
            show_y_guides=False,
            y_label_rotation=0,  # 倾斜y轴标签
            print_values=True,
            print_zeroes=False,
            show_legend=True,
            legend_at_bottom=False,
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            fill=True,
        )
        chart.x_labels = x
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def pie(self, 
            weights={'A': 1, 'B': 2, 'C': 3},
            title='Suluoya-svg',
            print_values=True,
            show_legend=True,
            legend_at_bottom=False,
            value_formatter=lambda x: "%.2f" % x,
            human_readable=True,
            inner_radius=0.6,  # 环图
            half_pie=False):
        from pygal import Pie
        chart = Pie(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title=title,
            print_values=print_values,
            show_legend=show_legend,
            legend_at_bottom=legend_at_bottom,
            value_formatter=value_formatter,
            human_readable=human_readable,
            inner_radius=inner_radius,  # 环图
            half_pie=half_pie,)
        for i, j in weights.items():
            chart.add(i, j)
        return chart

    def scatter(self, 
                data={'A': [(1, 2), (3, 4)], 'B': [(12, 24), (33, 44)], 'C': [(11, 2), (33, 44)]},
                title='Suluoya-scatter',
                x_title="I'm xlabel",
                y_title="I'm ylabel",
                show_x_labels=True,
                show_x_guide=False,
                show_y_labels=True,
                show_y_guides=False,
                print_values=False,
                print_zeroes=False,
                show_legend=True,
                legend_at_bottom=False,
                interpolation='cubic',  # 折线图平滑处理
                human_readable=True,
                stroke=False
                ):
        from pygal import XY
        chart = XY(
            width=self.width,
            height=self.height,
            spacing=self.spacing,
            margin=self.margin,
            style=self.style,
            title=title,
            x_title=x_title,
            y_title=y_title,
            show_x_labels=show_x_labels,
            show_x_guide=show_x_guide,
            show_y_labels=show_y_labels,
            show_y_guides=show_y_guides,
            print_values=print_values,
            print_zeroes=print_zeroes,
            show_legend=show_legend,
            legend_at_bottom=legend_at_bottom,
            interpolation=interpolation,  # 折线图平滑处理
            human_readable=human_readable,
            stroke=stroke,  # 不连线
        )
        for i, j in data.items():
            chart.add(i, j)
        return chart

    def save(self, chart, path='render'):
        chart.render_to_file(path+'.svg')


if __name__ == '__main__':
    s = svg_charts()
    b = s.scatter()
    s.save(chart=b)







import string
from zhon.hanzi import punctuation
import pdfplumber
from jieba import lcut
from jieba import add_word
from tqdm import tqdm
import PySimpleGUI as sg
import os
import csv
pun = string.punctuation+punctuation
n = True


def what(word):
    if word in pun:
        return '标点符号'
    elif word >= u'\u4e00' and word <= u'\u9fa5':
        return '中文'
    elif word.isdigit():
        return '数字'
    elif word.isalpha():
        return '英文'
    else:
        return '其他'


def sp(sentence):
    return [dict(zip(lcut(j), map(what, lcut(j)))) for j in sentence.split(' ') if j != '']


def calculate(file_name='bohaibank.pdf',
              target={'数字化': 0, '数据化': 0, '数字驱动': 0},
              what_dict={'标点符号': 0, '中文': 0, '数字': 0, '英文': 0, '其他': 0},
              path=None,save_path=None):
    global n
    with pdfplumber.open(path+'/'+file_name) as pdf:
        pages = pdf.pages
        word_number = 0
        print('\n'+file_name)
        for page in tqdm(pages):
            try:
                index = page.extract_text().split('\n')[:-1]
                data = map(sp, index)
                for i in data:
                    for j in i:
                        for key, value in j.items():
                            word_number += len(key)
                            what_dict[value] += 1
                            if value in target:
                                target[value] += 1
            except:
                pass
    target['总字数'] = word_number
    target.update(what_dict)
    target['银行'] = file_name

    with open(f'{save_path}/result.csv', 'a+', newline='') as f:
        f_csv = csv.DictWriter(f, list(target.keys()))
        if n:
            f_csv.writeheader()
            n = False
        f_csv.writerow(target)
    return target


keywords = '''
数字化
数据化
数字驱动
数据驱动
数据中心
数据应用
数据运用
数据挖掘
数据治理
大数据
数据治理委员会
首席数据官
数据仓库
数据集市
数据库
数据中台
数据平台
数据战略
数据管理
数据标准
信息安全
数据价值
数据质量
数据资产
可视化
数据分析
风险数据
数据管控数据模型
数据技术
数据支撑
数据支持
金融科技部
金融科技委员会
信息科技部
大数据部
数据管理和应用中心
'''


def gui():
    global keywords
    keywords=keywords[1:]
    sg.theme('BlueMono')
    layout = [
        [sg.FolderBrowse('选择文件夹', key='path')],
        [sg.Text('关键词')],
        [sg.Multiline(default_text=keywords, key='keywords',)],
        [sg.Text('类型')],
        [sg.Checkbox(text='标点符号', default=True),
         sg.Checkbox(text='中文', default=True),
         sg.Checkbox(text='数字', default=True),
         sg.Checkbox(text='英文', default=True),
         sg.Checkbox(text='其他', default=True), ],
        [sg.FolderBrowse('保存路径', key='save_path')],
        [sg.Button('戳这里ヾ(≧▽≦*)o')],
    ]
    window = sg.Window('( •̀ ω •́ )✧', layout)
    event,values = window.read()
    if event == 'Cancel' or event is None:
        window.close()
    keywords = values['keywords'][:-1].split('\n')
    target = {}
    for key in keywords:
        target[key] = 0
    what_dict = {}
    if values[0]:
        what_dict['标点符号'] = 0
    if values[1]:
        what_dict['中文'] = 0
    if values[2]:
        what_dict['数字'] = 0
    if values[3]:
        what_dict['英文'] = 0
    if values[4]:
        what_dict['其他'] = 0
    path = values['path']
    save_path = values['save_path']
    window.close()
    return target, what_dict, path, save_path


def pdf_statistics():
    
    target, what_dict, path, save_path = gui()
    while save_path == '':
        print('保存路径？？？？？？？？？？？？？？')
        target, what_dict, path, save_path = gui()
    while path == '':
        print('路径？？？？？？？？？？？？？？')
        target, what_dict, path, save_path = gui()
    for i in target:
        add_word(i)
    filenames = os.listdir(path)
    targets = []

    for filename in filenames:
        result = calculate(file_name=filename,
                           target=target, what_dict=what_dict,
                           path=path,save_path=save_path)
        targets.append(result)

    print('\n(*≧︶≦))(￣▽￣* )ゞ')


if __name__ == '__main__':
    target, what_dict, path, save_path = gui()
    while save_path == '':
        print('保存路径？？？？？？？？？？？？？？')
        target, what_dict, path, save_path = gui()
    while path == '':
        print('路径？？？？？？？？？？？？？？')
        target, what_dict, path, save_path = gui()
    for i in target:
        add_word(i)
    filenames = os.listdir(path)
    targets = []

    for filename in filenames[0:2]:
        result = calculate(file_name=filename,
                           target=target, what_dict=what_dict,
                           path=path,save_path=save_path)
        targets.append(result)

    print('\n(*≧︶≦))(￣▽￣* )ゞ')



import time
import pretty_errors
import pandas as pd
import baostock as bs
from tqdm import tqdm
import datetime
import sys
import os

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from .GetDate import GetDate
    from ..log.SlyLog import slog
    from ..log.SlyLog import sprint
except:
    from GetDate import GetDate
    from log.SlyLog import slog
    from log.SlyLog import sprint


class StockData(object):

    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_date='2020-12-01', end_date='2020-12-31',
                 frequency="w",
                 cache=False):
        self.cache = cache
        if self.cache == True:
            try:
                os.makedirs('./cache')
            except:
                pass
        self.sprint = sprint()
        self.slog = slog('stock info')
        self.start_date = start_date
        self.end_date = end_date
        self.frequency = frequency
        if type(names) == str:
            self.names = [names]
        elif type(names) == list:
            self.names = names
        self.sprint.hide()
        bs.login()
        self.sprint.show()
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)

    @property
    def stock_pair(self):
        self.sprint.pink('getting stock pair...')
        lists = []
        for i in tqdm(self.names):
            rs = bs.query_stock_industry()
            rs = bs.query_stock_basic(code_name=i)
            industry_list = []
            while (rs.error_code == '0') & rs.next():
                industry_list.append(rs.get_row_data())
            info = pd.DataFrame(industry_list, columns=rs.fields)
            lists.append(info)
        df = pd.concat(lists)
        stocks = {}
        for i, j in df[['code', 'code_name', 'ipoDate']].iterrows():
            if j[2] >= self.start_date:
                self.sprint.red(
                    f"{j[1]}'s ipo date is {j[2]},which is after {self.start_date}.\nPlease check your start date!")
                raise ValueError(
                    "The start date should be after the ipo date!")
            stocks[j[1]] = j[0]
            self.slog.log(f'{j[1]}——{j[0]}', mode=1)
        return stocks  # {'贵州茅台': 'sh.600519', '隆基股份': 'sh.601012'}

    @property
    def stock_data(self, info="date,code,open,close,volume,amount,adjustflag,turn,pctChg"):

        stock_pair = self.stock_pair
        codes = list(stock_pair.values())
        # 交换stock_pair键值对
        Name = dict(zip(stock_pair.values(), stock_pair.keys()))
        lists = []
        self.sprint.yellow('getting stock data...')
        for i in tqdm(codes):
            rs = bs.query_history_k_data_plus(i, info,
                                              start_date=self.start_date, end_date=self.end_date,
                                              frequency=self.frequency, adjustflag="3")
            data_list = []
            while (rs.error_code == '0') & rs.next():
                data_list.append(rs.get_row_data())
            result = pd.DataFrame(data_list, columns=rs.fields)
            result['name'] = Name[i]
            lists.append(result)
        df = pd.concat(lists)
        df = df[['date', 'name', 'code', 'open', 'close',
                 'volume', 'amount', 'adjustflag', 'turn', 'pctChg']]
        df=df.apply(pd.to_numeric,errors='ignore')
        if self.cache == True:
            df.to_csv(f'cache\\{self.names}.csv', encoding='utf8', index=False)
        return (codes,  # ['sh.600519', 'sh.601012']
                stock_pair,  # {'贵州茅台': 'sh.600519', '隆基股份': 'sh.601012'}
                df)

    def quit(self):
        self.sprint.hide()
        bs.logout()
        self.sprint.show()


class HolidayStockData(object):

    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_date='20200101',
                 end_date='20210101',
                 frequency='d',
                 holiday='国庆节', before=-21, after=21,
                 cache=False):
        self.cache = cache
        if self.cache == True:
            try:
                os.makedirs('./cache')
            except:
                pass
        self.sprint = sprint()
        self.slog = slog('holiday stock info')
        self.holiday = holiday
        self.before = before
        self.after = after
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        self.frequency = frequency
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)
        self.slog.log(f'holiday: {self.holiday}', mode=1)

    @property
    def HolidayNearbyDate(self):
        gd = GetDate(start=self.start_date, end=self.end_date)
        Date = gd.Date
        Date = Date[Date['holiday'] == self.holiday]
        if Date.empty == True:
            self.sprint.red(
                f'There are no festivals during {self.start_date}--{self.end_date}')
            raise ValueError('There are no festivals during this period!')
        start = []
        year = 0
        for i, j in Date.iterrows():
            if j[6] != year:
                start.append(str(j[0])[0:10])
            year = j[6]
        end = []
        year = list(Date['year'])[0]
        lastdate = None  # !!!!!!!!!!!!!!!!!!!!!!!!
        for i, j in Date.iterrows():
            if j[6] != year:
                end.append(str(lastdate)[0:10])
            year = j[6]
            lastdate = j[0]
        end.append(str(list(Date['date'])[-1])[0:10])

        def func(date, num):
            d = datetime.datetime.strptime(date, '%Y-%m-%d')
            delta = datetime.timedelta(days=num)
            d = d+delta
            return d.strftime('%Y-%m-%d')
        startbefore = []
        for i in start:
            startbefore.append(func(i, self.before))
        endafter = []
        for i in end:
            endafter.append(func(i, self.after))
        df = pd.DataFrame([startbefore, start, end, endafter], index=[
            'start before', 'start', 'end', 'end after'])
        df = df.T
        return df

    @property
    def HolidayNearbyData(self):
        date = self.HolidayNearbyDate
        lens = date.index.stop
        result = []
        global StockData
        for i in range(0, lens):
            start_date = list(date['start before'])[i]
            end_date = list(date['end after'])[i]
            StockData = StockData(
                names=self.names, start_date=start_date, end_date=end_date, frequency=self.frequency)
            data = StockData.stock_data
            result.append(data[2])
        df = pd.concat(result)
        StockData.quit()
        self.sprint.show()
        if self.cache == True:
            df.to_csv(f'cache\\{self.names}.csv', encoding='utf8', index=False)
        return (data[0], data[1], df)


class ConstituentStock(object):
    def __init__(self,save=True):
        self.bs = bs
        self.sprint = sprint()
        self.slog = slog('constituent stock info')
        self.sprint.hide()
        self.bs.login()
        self.sprint.show()
        self.save = save
        if self.save:
            try:
                os.makedirs('./result')
            except:
                pass

    def quit(self):
        self.sprint.hide()
        self.bs.logout()
        self.sprint.show()

    def StockIndustry(self, names=None):
        if type(names) is not list:
            names = [names]
        rs = self.bs.query_stock_industry()
        industry_list = []
        while (rs.error_code == '0') & rs.next():
            industry_list.append(rs.get_row_data())
        result = pd.DataFrame(industry_list, columns=rs.fields)
        if names != None:
            lists = []
            for name in names:
                x = result[result['code_name'] == name]
                lists.append(x)
                for i, j in x.iterrows():
                    for k in j:
                        self.slog.log(k, mode=2)
                    self.slog.log('')

        df = pd.concat(lists)
        if self.save:
            df.to_csv('result\\industry.csv', index=False, encoding='utf8')
        self.quit()
        return df

    # 上证50成分股
    @property
    def sz50(self):
        rs = bs.query_sz50_stocks()
        sz50_stocks = []
        while (rs.error_code == '0') & rs.next():
            sz50_stocks.append(rs.get_row_data())
        result = pd.DataFrame(sz50_stocks, columns=rs.fields)
        if self.save:
            result.to_csv('result\\sz50.csv', index=False, encoding='utf8')
        self.quit()
        return result

    # 沪深300成分股
    @property
    def hs300(self):
        rs = bs.query_hs300_stocks()
        hs300_stocks = []
        while (rs.error_code == '0') & rs.next():
            hs300_stocks.append(rs.get_row_data())
        result = pd.DataFrame(hs300_stocks, columns=rs.fields)
        if self.save:
            result.to_csv('result\\hs300.csv', index=False, encoding='utf8')
        self.quit()
        return result

    # 中证500成分股
    @property
    def zz500(self):
        rs = bs.query_zz500_stocks()
        zz500_stocks = []
        while (rs.error_code == '0') & rs.next():
            zz500_stocks.append(rs.get_row_data())
        result = pd.DataFrame(zz500_stocks, columns=rs.fields)
        if self.save:
            result.to_csv('result\\zz500.csv', index=False, encoding='utf8')
        self.quit()
        return result


class StockAbility(object):

    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_year=2018, start_quater=1,
                 end_year=2019, end_quater=4):
        self.sprint = sprint()
        self.slog = slog('stock ability')
        self.sprint.red(
            'Please make sure that all the data has already existed!')
        self.names = names
        self.start_year = start_year
        self.end_year = end_year
        self.start_quater = start_quater
        self.end_quater = end_quater
        self.slog.log(
            f'time: {self.start_year},{self.start_quater} ~~ {self.end_year},{self.end_quater}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        Range = []
        if end_year-start_year >= 2:
            for i in range(start_quater, 5):
                Range.append([start_year, i])
            for i in range(start_year+1, end_year):
                for j in range(1, 5):
                    Range.append([i, j])
            for i in range(1, end_quater+1):
                Range.append([end_year, i])
        elif end_year == start_year:
            for i in range(start_quater, end_quater+1):
                Range.append([end_year, i])
        else:
            for i in range(start_quater, 5):
                Range.append([start_year, i])
            for i in range(1, end_quater+1):
                Range.append([end_year, i])
        self.Range = Range
        self.StockData = StockData(names=self.names)
        self.stock_pair = self.StockData.stock_pair

    # 盈利能力
    @property
    def profit(self):
        intro = '''
        季频盈利能力
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        roeAvg	净资产收益率(平均)(%)	归属母公司股东净利润/[(期初归属母公司股东的权益+期末归属母公司股东的权益)/2]*100%
        npMargin	销售净利率(%)	净利润/营业收入*100%
        gpMargin	销售毛利率(%)	毛利/营业收入*100%=(营业收入-营业成本)/营业收入*100%
        netProfit	净利润(元)	
        epsTTM	每股收益	归属母公司股东的净利润TTM/最新总股本
        MBRevenue	主营营业收入(元)	
        totalShare	总股本	
        liqaShare	流通股本
        '''
        self.slog.log(intro)
        profit = []
        self.sprint.blue('getting profit data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                profit_list = []
                profit_list.append(i)
                profit_list.append(k[0])
                profit_list.append(k[1])
                rs_profit = bs.query_profit_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_profit.error_code == '0') & rs_profit.next():
                    profit_list = profit_list+rs_profit.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_profit.fields
                result_profit = pd.DataFrame([profit_list], columns=columns)
                profit.append(result_profit)
        df = pd.concat(profit)
        return df

    # 营运能力
    @property
    def operation(self):
        intro = '''
        季频营运能力
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        NRTurnRatio	应收账款周转率(次)	营业收入/[(期初应收票据及应收账款净额+期末应收票据及应收账款净额)/2]
        NRTurnDays	应收账款周转天数(天)	季报天数/应收账款周转率(一季报：90天，中报：180天，三季报：270天，年报：360天)
        INVTurnRatio	存货周转率(次)	营业成本/[(期初存货净额+期末存货净额)/2]
        INVTurnDays	存货周转天数(天)	季报天数/存货周转率(一季报：90天，中报：180天，三季报：270天，年报：360天)
        CATurnRatio	流动资产周转率(次)	营业总收入/[(期初流动资产+期末流动资产)/2]
        AssetTurnRatio	总资产周转率	营业总收入/[(期初资产总额+期末资产总额)/2]
        '''
        self.slog.log(intro)
        operation = []
        self.sprint.blue('getting operation data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                operation_list = []
                operation_list.append(i)
                operation_list.append(k[0])
                operation_list.append(k[1])
                rs_operation = bs.query_operation_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_operation.error_code == '0') & rs_operation.next():
                    operation_list = operation_list+rs_operation.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_operation.fields
                result_operation = pd.DataFrame(
                    [operation_list], columns=columns)
                operation.append(result_operation)
        df = pd.concat(operation)
        return df

    # 成长能力
    @property
    def growth(self):
        intro = '''
        季频成长能力
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        YOYEquity	净资产同比增长率	(本期净资产-上年同期净资产)/上年同期净资产的绝对值*100%
        YOYAsset	总资产同比增长率	(本期总资产-上年同期总资产)/上年同期总资产的绝对值*100%
        YOYNI	净利润同比增长率	(本期净利润-上年同期净利润)/上年同期净利润的绝对值*100%
        YOYEPSBasic	基本每股收益同比增长率	(本期基本每股收益-上年同期基本每股收益)/上年同期基本每股收益的绝对值*100%
        YOYPNI	归属母公司股东净利润同比增长率	(本期归属母公司股东净利润-上年同期归属母公司股东净利润)/上年同期归属母公司股东净利润的绝对值*100%
        '''
        self.slog.log(intro)
        growth = []
        self.sprint.blue('getting growth data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                growth_list = []
                growth_list.append(i)
                growth_list.append(k[0])
                growth_list.append(k[1])
                rs_growth = bs.query_growth_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_growth.error_code == '0') & rs_growth.next():
                    growth_list = growth_list+rs_growth.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_growth.fields
                result_growth = pd.DataFrame([growth_list], columns=columns)
                growth.append(result_growth)
        df = pd.concat(growth)
        return df

    # 偿债能力
    @property
    def balance(self):
        intro = '''
        季频偿债能力
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        currentRatio	流动比率	流动资产/流动负债
        quickRatio	速动比率	(流动资产-存货净额)/流动负债
        cashRatio	现金比率	(货币资金+交易性金融资产)/流动负债
        YOYLiability	总负债同比增长率	(本期总负债-上年同期总负债)/上年同期中负债的绝对值*100%
        liabilityToAsset	资产负债率	负债总额/资产总额
        assetToEquity	权益乘数	资产总额/股东权益总额=1/(1-资产负债率)        
        '''
        self.slog.log(intro)
        balance = []
        self.sprint.blue('getting balance data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                balance_list = []
                balance_list.append(i)
                balance_list.append(k[0])
                balance_list.append(k[1])
                rs_balance = bs.query_balance_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_balance.error_code == '0') & rs_balance.next():
                    balance_list = balance_list+rs_balance.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_balance.fields
                result_balance = pd.DataFrame([balance_list], columns=columns)
                balance.append(result_balance)
        df = pd.concat(balance)
        return df

    # 现金流量
    @property
    def cash_flow(self):
        intro = '''
        季频现金流量
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        CAToAsset	流动资产除以总资产	
        NCAToAsset	非流动资产除以总资产	
        tangibleAssetToAsset	有形资产除以总资产	
        ebitToInterest	已获利息倍数	息税前利润/利息费用
        CFOToOR	经营活动产生的现金流量净额除以营业收入	
        CFOToNP	经营性现金净流量除以净利润	
        CFOToGr	经营性现金净流量除以营业总收入
        '''
        self.slog.log(intro)
        cash_flow = []
        self.sprint.blue('getting cash flow data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                cash_flow_list = []
                cash_flow_list.append(i)
                cash_flow_list.append(k[0])
                cash_flow_list.append(k[1])
                rs_cash_flow = bs.query_cash_flow_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_cash_flow.error_code == '0') & rs_cash_flow.next():
                    cash_flow_list = cash_flow_list+rs_cash_flow.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_cash_flow.fields
                result_cash_flow = pd.DataFrame(
                    [cash_flow_list], columns=columns)
                cash_flow.append(result_cash_flow)
        df = pd.concat(cash_flow)
        return df

    # 杜邦指数
    @property
    def dupont_data(self):
        intro = '''
        季频杜邦指数
        code	证券代码	
        pubDate	公司发布财报的日期	
        statDate	财报统计的季度的最后一天, 比如2017-03-31, 2017-06-30	
        dupontROE	净资产收益率	归属母公司股东净利润/[(期初归属母公司股东的权益+期末归属母公司股东的权益)/2]*100%
        dupontAssetStoEquity	权益乘数，反映企业财务杠杆效应强弱和财务风险	平均总资产/平均归属于母公司的股东权益
        dupontAssetTurn	总资产周转率，反映企业资产管理效率的指标	营业总收入/[(期初资产总额+期末资产总额)/2]
        dupontPnitoni	归属母公司股东的净利润/净利润，反映母公司控股子公司百分比。如果企业追加投资，扩大持股比例，则本指标会增加。	
        dupontNitogr	净利润/营业总收入，反映企业销售获利率	
        dupontTaxBurden	净利润/利润总额，反映企业税负水平，该比值高则税负较低。净利润/利润总额=1-所得税/利润总额	
        dupontIntburden	利润总额/息税前利润，反映企业利息负担，该比值高则税负较低。利润总额/息税前利润=1-利息费用/息税前利润
        dupontEbittogr	息税前利润/营业总收入，反映企业经营利润率，是企业经营获得的可供全体投资人（股东和债权人）分配的盈利占企业全部营收收入的百分比
        '''
        self.slog.log(intro)
        dupont_data = []
        self.sprint.blue('getting dupont data')
        for i, j in self.stock_pair.items():
            self.sprint.cyan(i)
            for k in tqdm(self.Range):
                dupont_data_list = []
                dupont_data_list.append(i)
                dupont_data_list.append(k[0])
                dupont_data_list.append(k[1])
                rs_dupont_data = bs.query_dupont_data(
                    code=j, year=k[0], quarter=k[1])
                while (rs_dupont_data.error_code == '0') & rs_dupont_data.next():
                    dupont_data_list = dupont_data_list+rs_dupont_data.get_row_data()
                columns = ['name', 'year', 'quater']
                columns = columns+rs_dupont_data.fields
                result_dupont_data = pd.DataFrame(
                    [dupont_data_list], columns=columns)
                dupont_data.append(result_dupont_data)
        df = pd.concat(dupont_data)
        return df

    @property
    def AllAbility(self):
        profit = self.profit
        operation = self.operation
        growth = self.growth
        balance = self.balance
        cash_flow = self.cash_flow
        dupont_data = self.dupont_data
        lists = [growth, balance, cash_flow, dupont_data]
        df = pd.merge(profit, operation, how='outer', on=[
            'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
        for i in lists:
            df = pd.merge(df, i, how='outer', on=[
                'code', 'name', 'year', 'quater', 'pubDate', 'statDate'])
        return df


if __name__ == '__main__':
    #StockData = StockData(names=['d'],cache=True)
    # print(StockData.stock_pair)
    # HolidayStockData=HolidayStockData()
    # print(HolidayStockData.HolidayNearbyData)
    cs = ConstituentStock()
    print(cs.StockIndustry(names=['五粮液', '贵州茅台']))



class cute_charts(object):
    def __init__(self,
                 width="1440px",
                 height="720px"):
        self.width = width
        self.height = height

    def line(self, name='line',
             x=["a", "b", "c", "d", "e", "f", "g"],
             y={"series-A": [57, 134, 137, 129, 145, 60, 49],
                "series-B": [114, 55, 27, 101, 125, 27, 105]},
             x_label="I'm xlabel",
             y_label="I'm ylabel",):
        from cutecharts.charts import Line
        chart = Line(title=name,
                     width=self.width,
                     height=self.height)
        chart.set_options(
            labels=x,
            x_label=x_label,
            y_label=y_label,
            legend_pos="upRight"
        )
        for i, j in y.items():
            chart.add_series(i, j)
        return chart

    def bar(self, name='bar',
            x=["a", "b", "c", "d", "e", "f", "g"],
            y={"series-A": [57, 134, 137, 129, 145, 60, 49],
               "series-B": [114, 55, 27, 101, 125, 27, 105]},
            x_label="I'm xlabel",
            y_label="I'm ylabel",):
        from cutecharts.charts import Bar
        chart = Bar(title=name,
                    width=self.width,
                    height=self.height)
        chart.set_options(
            labels=x,
            x_label=x_label,
            y_label=y_label,
        )
        for i, j in y.items():
            chart.add_series(i, j)
        return chart

    def pie(self, name='pie',
            weights={'A': 1, 'B': 2, 'C': 3}):
        from cutecharts.charts import Pie
        chart = Pie(title=name,
                    width=self.width,
                    height=self.height)
        chart.set_options(labels=list(weights.keys()))
        chart.add_series(list(weights.values()))
        return chart

    def scatter(self, name='scatter',
                data={"series-A": [(79, 123), (76, 128), (84, 125), ],
                      "series-B": [(112, 20), (119, 113), (76, 126), (30, 149), ]
                      },
                x_label="I'm xlabel",
                y_label="I'm ylabel",
                dot_size=1.5,  # 点大小
                x_tick_count=3,  # X 轴刻度分割段数
                y_tick_count=3,  # Y 轴刻度分割段数
                line=False  # 是否连线
                ):
        from cutecharts.charts import Scatter
        chart = Scatter(title=name,
                        width=self.width,
                        height=self.height)
        chart.set_options(dot_size=dot_size,
                          x_tick_count=x_tick_count, y_tick_count=y_tick_count,
                          is_show_line=line,
                          x_label=x_label, y_label=y_label)
        for i, j in data.items():
            chart.add_series(i, j)
        return chart

    def save(self, chart, path='render'):
        chart.render(path+'.html')


if __name__ == '__main__':
    cc = cute_charts()
    l = cc.line()
    cc.save(l)



import time


class sprint(object):
    
    def __init__(self):
        '''
        red, green, yellow, blue, magenta紫, cyan青, white.
        '''
        from termcolor import colored
        self.colored = colored

    def red(self, content='Suluoya'):
        print(self.colored(content, 'red'))

    def green(self, content='Suluoya'):
        print(self.colored(content, 'green'))

    def yellow(self, content='Suluoya'):
        print(self.colored(content, 'yellow'))

    def blue(self, content='Suluoya'):
        print(self.colored(content, 'blue'))

    def magenta(self, content='Suluoya'):
        print(self.colored(content, 'magenta'))

    def cyan(self, content='Suluoya'):
        print(self.colored(content, 'cyan'))

    def pink(self, contents='Suluoya'):
        print(f'\033[1;35;40m{contents}')
        print(self.colored('', 'white'), end='')

    def black(self, contents='Suluoya'):
        print(f'\033[1;30;40m{contents}')
        print(self.colored('', 'white'), end='')
    
    def hide(self):
        print(f'\033[0;30;40m',end='')
    
    def show(self):
        print(self.colored('', 'white'), end='')


class slog(sprint):
    def __init__(self,filename='log'):
        import os
        try:
            os.makedirs('./log')
        except:
            pass
        self.filename = filename
        self.Time = time.asctime(time.localtime(time.time()))
        with open(f'log\\{self.filename}.log', 'w', encoding='utf8') as f:
            f.write(self.Time+'\n')
            f.write("(｡･∀･)ﾉﾞ嗨!\n\n")
    def log(self,content='Suluoya',mode=0):
        with open(f'log\\{self.filename}.log', 'a', encoding='utf8') as f:
            if mode == 0:
                f.write(f'\n{content}\n')
            elif mode == 1:
                f.write(content+'\n')
            elif mode == 2:
                f.write(content+' ')





class SlyDataFrame(object):
    
    def __init__(self,df=None):
        self.df = df
    
    @property    
    def report(self):
        import pandas_profiling
        a = pandas_profiling.ProfileReport(self.df)
        a.to_file('report.html')
    
    @property
    def gui(self):
        from pandasgui import show
        show(self.df)
    
    @property
    def sweetviz(self):
        import sweetviz as sv 
        x=sv.analyze(self.df)
        x.show_html()
        




__name__ = "Suluoya"
__author__ = 'Suluoya'
__all__ = ['Markovitz', 'GetGoodStock', 'GetData', 'GetDate', 'gui']

from .GetGoodStock import *
from .GetDate import *
from .GetData import *
from .Markovitz import *
from .gui import *
import pretty_errors



from lunar_python.util import HolidayUtil
import pandas as pd
import time


class GetDate(object):
    def __init__(self, start='20000101', end=None):
        if end == None:
            end = time.strftime("%Y%m%d", time.localtime())
        self.end = end
        self.start = start
        date = pd.Series(pd.date_range(start=start, end=end, freq='D'))
        self.date = date

    @property
    def holiday(self):
        series = pd.Series(
            [HolidayUtil.getHoliday(str(i)[0:10]) for i in self.date])

        def func1(x):
            if x != None:
                name = str(x)[11:-11]
                if '调休' in name:
                    return None
                if name == '国庆中秋':
                    name = '国庆节'
                if str(x)[-10:] == '2009-10-03':
                    name = '国庆节'
                return name
        holiday = series.map(func1)

        def func2(x):
            if x != None:
                return str(x)[-11:]
        holidaystart = series.map(func2)
        return (holiday, holidaystart)

    @property
    def day(self):
        return pd.Series([i.day for i in self.date])

    @property
    def week(self):
        return pd.Series([i.week for i in self.date])

    @property
    def month(self):
        return pd.Series([i.month for i in self.date])

    @property
    def year(self):
        return pd.Series([i.year for i in self.date])

    @property
    def weekofyear(self):
        return pd.Series([i.weekofyear for i in self.date])

    @property
    def dayofyear(self):
        return pd.Series([i.dayofyear for i in self.date])

    @property
    def Date(self):
        Date = pd.DataFrame()
        Date['date'] = self.date
        Date['holiday'] = self.holiday[0]
        Date['holidaystart'] = self.holiday[1]
        Date['day'] = self.day
        Date['week'] = self.week
        Date['month'] = self.month
        Date['year'] = self.year
        Date['dayofyear'] = self.dayofyear
        Date['weekofyear'] = self.weekofyear
        return Date



if __name__ == '__main__':
    gd=GetDate(start='20000101',end=None)
    print(gd.start,gd.end,gd.date)


import pretty_errors
import pandas as pd
from tqdm import tqdm
from tqdm import trange
import numpy as np
import random
import scipy.optimize as sco
import time
import sys
import os
sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from ..chart.svg_charts import svg_charts
    from .GetData import StockData
    from .GetData import HolidayStockData
    from ..log.SlyLog import slog
    from ..log.SlyLog import sprint
except:
    from chart.svg_charts import svg_charts
    from GetData import StockData
    from GetData import HolidayStockData
    from log.SlyLog import slog
    from log.SlyLog import sprint


class calculate(object):
    def __init__(self):
        pass

    def correlation(self, x, y):
        if type(x) == list:
            x = np.array(x)
        if type(y) == list:
            y = np.array(y)
        return (np.dot([xi - np.mean(x) for xi in x], [yi - np.mean(y) for yi in y])/(len(x)-1))/((x.std())*(y.std()))

    def weight(self, lists=['A', 'B', 'C'], mode='dict'):
        if mode == 'dict':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return rand
        elif mode == 'list':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return list(rand.values())


class Markovitz(calculate):
    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_date='2019-01-01',
                 end_date='2020-01-01',
                 frequency="w",
                 holiday=False,
                 holiday_name='国庆节',
                 before=-21, after=21,
                 no_risk_rate=0.0185
                 ):
        '''
        马科维兹投资组合
        '''
        self.svg_charts = svg_charts()
        self.sprint = sprint()
        self.slog = slog('Markovitz ')
        self.frequency = frequency
        if frequency == 'd':    
            self.no_risk_rate = no_risk_rate / 365
        elif frequency == 'w':
            self.no_risk_rate = no_risk_rate / 52
        elif frequency == 'm':
            self.no_risk_rate = no_risk_rate / 12
        else:
            raise ValueError('frequency should be "d" or "w" or "m"!')
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)
        self.slog.log(
            f'risk-free interest rate: {round(self.no_risk_rate,6)}', mode=1)
        try:
            os.makedirs('./picture')
        except:
            pass
        cache = False
        while cache == False:
            try:
                self.stock_data = pd.read_csv(f'cache\\{str(self.names)}.csv')
                self.sprint.cyan('Use cache data to calculate!')
                self.codes = None
                self.stock_pair = None
                break
            except:
                self.sprint.cyan('No cache data found!')
                cache = True
                self.sprint.red(
                    'Please make sure that all of the stocks are in the market!')
                self.sprint.green('initializing...')
                if holiday == False:
                    self.StockData = StockData(names=self.names,
                                               start_date=self.start_date,
                                               end_date=self.end_date,
                                               frequency=self.frequency,
                                               cache=cache)
                    try:
                        self.codes, self.stock_pair, self.stock_data = self.StockData.stock_data
                        self.StockData.quit()
                    except:
                        raise ValueError('data get error, please retry!')
                else:
                    if holiday_name == None:
                        holiday_name = '国庆节'
                    self.holiday_name = holiday_name
                    start_date = self.start_date.replace('-', '')
                    end_date = self.end_date.replace('-', '')
                    self.HolidayStockData = HolidayStockData(names=names,
                                                             start_date=start_date,
                                                             end_date=end_date,
                                                             holiday=holiday_name,
                                                             frequency=frequency,
                                                             before=before, after=after,
                                                             cache=cache)
                    try:
                        self.codes, self.stock_pair, self.stock_data = self.HolidayStockData.HolidayNearbyData
                    except:
                        raise ValueError('data get error, please retry!')
                date = list(set(list(self.stock_data['date'])))
                data = {}
                for i in self.names:
                    data[i] = list(
                        self.stock_data[self.stock_data['name'] == i]['close'])
                line = self.svg_charts.line(x=date, data=data, title='close', x_title='date',
                                            y_title='close price', x_label_rotation=90, print_values=False)
                self.svg_charts.save(line, path="picture\\close")
                data = {}
                for i in self.names:
                    data[i] = list(
                        self.stock_data[self.stock_data['name'] == i]['pctChg'])
                line = self.svg_charts.line(x=date, data=data, title='pctChg', x_title='date',
                                            y_title='pctChg', x_label_rotation=90, print_values=False)
                self.svg_charts.save(line, path="picture\\pctChg")
                break

    def sharp(self, weights=[], stock_list=[]):
        if weights == []:
            weights = len(self.names) * [1. / len(self.names), ]
        if stock_list == []:
            stock_list = self.names
        df = self.stock_data[['pctChg', 'name']]
        covs = []
        for i in range(0, len(stock_list)):
            for j in stock_list[i:]:
                if stock_list[i] != j:
                    covs.append([stock_list[i], j])
        for i in covs:
            x = list(df[df['name'] == i[0]]['pctChg'])
            y = list(df[df['name'] == i[1]]['pctChg'])
            i.append(self.correlation(x, y))
        dic = {}
        rate = 0
        risk = 0
        c = 0
        rand = {}
        for i in stock_list:
            weight = weights[c]
            rand[i] = weight
            data = list(df[df['name'] == i]['pctChg'])
            Mean = np.mean(data)
            Std = np.std(data)
            rate += Mean*weight
            risk += (weight*Std)**2
            dic[i] = weight*Std
            c += 1
        for i in covs:
            risk += 2*i[2]*dic[i[0]]*dic[i[1]]
        sharp = (rate-self.no_risk_rate)/risk
        return {'sharp': sharp, 'rate': rate, 'risk': risk, 'weight': rand}

    def max_sharp(self, weights, *args):
        stock_list = args[0]
        self.slog.log(f'weights: {weights}', mode=1)
        return -self.sharp(weights=weights, stock_list=stock_list)['sharp']

    def portfolio(self, stock_list=[], accurate=True, number=500):
        if stock_list == []:
            stock_list = self.names
        self.sprint.magenta(f'building a portfolio for {stock_list}')
        try:
            os.makedirs('./result')
        except:
            pass
        if accurate == True:
            opts = sco.minimize(fun=self.max_sharp,
                                x0=len(stock_list) * [1. / len(stock_list), ],
                                method='SLSQP',
                                args=(stock_list,),
                                bounds=tuple((0, 1)
                                             for x in range(len(stock_list))),
                                constraints={'type': 'eq',
                                             'fun': lambda x: np.sum(x) - 1}
                                )
            count = 0
            result = {}
            ans = list(opts['x'])
            for i in stock_list:
                result[i] = ans[count]
                count += 1
            pie = self.svg_charts.pie(weights=result, title='weights-pie')
            self.svg_charts.save(pie, path='picture\\weights-pie')
            result = [result, -opts['fun']]
            self.slog.log(f'weights:{result[0]}\nsharp:{result[1]}')
            df = pd.DataFrame([result], columns=['weights', 'sharp'])
            df.to_csv('result\\accurate_result.csv',
                      index=False, encoding='utf8')
            return result
        else:
            lists = []
            for i in trange(0,number):
                weights = self.weight(lists=stock_list, mode='list')
                sharp = self.sharp(
                    weights=weights, stock_list=stock_list)
                lists.append(list(sharp.values()))
            df = pd.DataFrame(
                lists, columns=['sharp', 'rate', 'risk', 'weight'])
            df = df.sort_values('sharp', ascending=False)
            data = []
            for i, j in df.iterrows():
                data.append((j[1], j[2]))
            scatter = self.svg_charts.scatter(
                data={'': data}, title='sharp-scatter', x_title='rate of return', y_title='risk')
            self.svg_charts.save(scatter, path='picture\\sharp-scatter')
            weights = list(df['weight'])[0]
            sharp = list(df['sharp'])[0]
            pie = self.svg_charts.pie(weights=weights, title='weights-pie')
            self.svg_charts.save(pie, path='picture\\weights-pie')
            self.slog.log(f'weights:{weights}\nsharp:{sharp}')
            df.to_csv('result\\not_accurate_result.csv',
                      index=False, encoding='utf8')
            return df


if __name__ == '__main__':
    Markovitz = Markovitz(names=['隆基股份', '五粮液', '贵州茅台'], start_date='2019-12-01',
                          end_date='2020-12-31',
                          frequency='w',
                          holiday=False,
                          )
    print(Markovitz .portfolio(accurate=False))
